package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.*;
import com.diagbot.entity.MappingConfig;
import com.diagbot.entity.SysUser;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.MappingConfigService;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.*;
import com.google.common.collect.Lists;
import lombok.val;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/2/23 15:41
 */
@Component
public class KlConceptStaticFacade {

    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private ConceptInfoFacade conceptInfoFacade;
    @Autowired
    private SysUserFacade sysUserFacade;
    @Autowired
    MrFacade mrFacade;
    @Autowired
    private AssembleFacade assembleFacade;
    @Autowired
    private MappingConfigFacade mappingConfigFacade;
    private MappingConfigService mappingConfigService;


    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();
        List<ClassicCaseIndexDTO> cases = Lists.newLinkedList();
        RespDTO<List<ClassicCaseIndexDTO>> caseDTO = RespDTO.onSuc(new ArrayList<>(0));
        RespDTO<List<StaticKnowledgeIndexDTO>> respDTO = RespDTO.onSuc(new ArrayList<>(0));

        if (staticKnowledgeIndexVO.getTypes().contains(0) ||
                staticKnowledgeIndexVO.getTypes().contains(11) ||
                staticKnowledgeIndexVO.getTypes().isEmpty()) {
            caseDTO = cdssCoreClient.classicCaseIndex(staticKnowledgeIndexVO);
            if (staticKnowledgeIndexVO.getTypes().contains(11) && staticKnowledgeIndexVO.getTypes().size() > 1)
                staticKnowledgeIndexVO.getTypes().remove(staticKnowledgeIndexVO.getTypes().indexOf(11));
        }
        if (!staticKnowledgeIndexVO.getTypes().contains(11))
            respDTO = cdssCoreClient.staticKnowledgeIndex(staticKnowledgeIndexVO);

        if (RespDTOUtil.respIsOK(respDTO)) {
            retList = respDTO.data;
            if (RespDTOUtil.respIsOK(caseDTO)) {
                for (ClassicCaseIndexDTO classicCase : caseDTO.data) {
                    StaticKnowledgeIndexDTO dto = new StaticKnowledgeIndexDTO();
                    BeanUtil.copyProperties(classicCase, dto);
                    retList.add(dto);
                }
            }
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }

        if (staticKnowledgeIndexVO.getHasInfo() != null && staticKnowledgeIndexVO.getHasInfo().equals(1)) {
            retList = retList.stream().filter(i -> i.getStatus().equals(1)).collect(Collectors.toList());
        }
        return retList;
    }

    /**
     * 收藏静态知识检索
     *
     * @param
     * @return
     */
    public List<StaticKnowledgeIndexDTO> CollectionStaticKnowledgeIndex() {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();
        RespDTO<List<ClassicCaseIndexDTO>> caseDTO = RespDTO.onSuc(new ArrayList<>(0));
        RespDTO<List<StaticKnowledgeIndexDTO>> respDTO = RespDTO.onSuc(new ArrayList<>(0));

        caseDTO = cdssCoreClient.collectionClassicCaseIndex();
        respDTO = cdssCoreClient.collectionStaticKnowledgeIndex();

        if (RespDTOUtil.respIsOK(respDTO)) {
            retList = respDTO.data;
            if (RespDTOUtil.respIsOK(caseDTO)) {
                for (ClassicCaseIndexDTO classicCase : caseDTO.data) {
                    StaticKnowledgeIndexDTO dto = new StaticKnowledgeIndexDTO();
                    BeanUtil.copyProperties(classicCase, dto);
                    dto.setCollectionStatus(classicCase.getCollectionStatus());
                    retList.add(dto);
                }
            }
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }

        return retList;
    }

    /**
     * 经典病例检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<ClassicCaseIndexDTO> classicCaseIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<ClassicCaseIndexDTO> retList = Lists.newLinkedList();

        RespDTO<List<ClassicCaseIndexDTO>> respDTO = cdssCoreClient.classicCaseIndex(staticKnowledgeIndexVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            retList = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }

        if (staticKnowledgeIndexVO.getHasInfo() != null && staticKnowledgeIndexVO.getHasInfo().equals(1)) {
            retList = retList.stream().filter(i -> i.getStatus().equals(1)).collect(Collectors.toList());
        }
        return retList;
    }

    /**
     * 获取静态知识
     *
     * @param staticKnowledgeVO
     * @return
     */
    public StaticKnowledgeDTO getStaticKnowledge(StaticKnowledgeVO staticKnowledgeVO) {
        ScaleStaticAllVO scaleStaticAllVO = new ScaleStaticAllVO();
        if (staticKnowledgeVO.getType() == 8) {
            PushJoinVO mr = mrFacade.getMr(staticKnowledgeVO.getMrId());
            if (null != mr) {
                SearchData searchData = new SearchData();
                BeanUtil.copyProperties(mr, searchData);
                //入参映射
                searchData = assembleFacade.assembleData(searchData);
                BeanUtil.copyProperties(searchData, scaleStaticAllVO);
            } else {
                scaleStaticAllVO.setHospitalId(-1L);
            }
        } else {
            scaleStaticAllVO.setHospitalId(-1L);
        }
        BeanUtil.copyProperties(staticKnowledgeVO, scaleStaticAllVO);
        RespDTO<StaticKnowledgeDTO> respDTO;
        if (staticKnowledgeVO.getType().equals(11))
            respDTO = cdssCoreClient.getClassicCaseInfo(scaleStaticAllVO);
        else
            respDTO = cdssCoreClient.getStaticKnowledge(scaleStaticAllVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            StaticKnowledgeDTO staticKnowledgeDTO = respDTO.data;
            return staticKnowledgeDTO;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
    }

    /**
     * 医院端获取静态知识（对接）
     *
     * @param staticKnowledgeHISVO
     * @return
     */
    public List<StaticKnowledgeHISDTO> getStaticKnowledgeForHIS(StaticKnowledgeHISVO staticKnowledgeHISVO) {

        List<StaticKnowledgeHISDTO> retList = Lists.newArrayList();

        //术语映射
        List<String> uniqueNameList = conceptInfoFacade.getUniqueNames(staticKnowledgeHISVO);
        if (ListUtil.isEmpty(uniqueNameList)) {
            return retList;
        }

        for (String uniqueName : uniqueNameList) {
            StaticKnowledgeVO staticKnowledgeVO = new StaticKnowledgeVO();
            BeanUtil.copyProperties(staticKnowledgeHISVO, staticKnowledgeVO);
            staticKnowledgeVO.setName(uniqueName);
            try {
                StaticKnowledgeDTO staticKnowledgeDTO = getStaticKnowledge(staticKnowledgeVO);
                if (staticKnowledgeDTO != null) {
                    StaticKnowledgeHISDTO staticKnowledgeHISDTO = new StaticKnowledgeHISDTO();
                    BeanUtil.copyProperties(staticKnowledgeDTO, staticKnowledgeHISDTO);
                    staticKnowledgeHISDTO.setHisName(staticKnowledgeHISVO.getHisName());
                    staticKnowledgeHISDTO.setHisDetailName(staticKnowledgeHISVO.getHisDetailName());
                    retList.add(staticKnowledgeHISDTO);
                }
            } catch (CommonException e) {
            }
        }
        return retList;
    }

    /**
     * 医院对接静态知识判断是否有静态知识
     *
     * @param staticKnowledgeHISVO
     * @return
     */
    public Boolean isExistForHIS(StaticKnowledgeHISVO staticKnowledgeHISVO) {
        List<StaticKnowledgeHISDTO> staticList = getStaticKnowledgeForHIS(staticKnowledgeHISVO);
        if (ListUtil.isEmpty(staticList)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    public IPage<KlConceptStaticDTO> getPage(KlConceptStaticPageVO klConceptStaticPageVO) {
        Page<KlConceptStaticDTO> page = null;
        RespDTO<Page<KlConceptStaticDTO>> respDTO = cdssCoreClient.getPage(klConceptStaticPageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        List<KlConceptStaticDTO> records = page.getRecords();
        List<Long> userIds = records.stream().map(KlConceptStaticDTO::getModifier)
                .distinct()
                .map(Long::parseLong)
                .collect(Collectors.toList());
        List<SysUser> users = sysUserFacade.list(new QueryWrapper<SysUser>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("id", userIds, false));
        if (ListUtil.isNotEmpty(users)) {
            Map<Long, String> userMap = users.stream().collect(Collectors.toMap(SysUser::getId, SysUser::getLinkman));
            records.forEach(record -> {
                if (userMap.containsKey(Long.valueOf(record.getModifier()))) {
                    record.setModifier(userMap.get(Long.valueOf(record.getModifier())));
                } else {
                    record.setModifier("");
                }
            });

        } else {
            records.forEach(record -> {
                record.setModifier("");
            });
        }
        page.setRecords(records);
        return page;
    }

    /**
     * 经典病例分页查询
     *
     * @param klClassicCasePageVO
     * @return
     */
    public IPage<KlClassicCaseDTO> getClassicCase(KlClassicCasePageVO klClassicCasePageVO) {
        Page<KlClassicCaseDTO> page = null;
        RespDTO<Page<KlClassicCaseDTO>> respDTO = cdssCoreClient.getClassicCase(klClassicCasePageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        List<KlClassicCaseDTO> records = page.getRecords();
        List<String> userIds = records.stream().map(KlClassicCaseDTO::getModifier)
                .distinct()
                .collect(Collectors.toList());
        RespDTO<Map<String, String>> userRespDTO = RespDTO.onSuc(sysUserFacade.getUserInfoByIds(userIds));
        if (RespDTOUtil.respIsNG(userRespDTO)) {
            return page;
        }
        Map<String, String> userMap = userRespDTO.data;
        records.forEach(record -> {
            if (userMap.containsKey(record.getModifier())) {
                record.setModifier(userMap.get(record.getModifier()));
            } else {
                record.setModifier("");
            }
        });
        page.setRecords(records);
        return page;
    }

    /**
     * 分页查询
     *
     * @param staticKnowledgeIndexPageVO
     * @return
     */
    public IPage<StaticKnowledgeIndexPageDTO> staticIndexPage(StaticKnowledgeIndexPageVO staticKnowledgeIndexPageVO) {
        Page<StaticKnowledgeIndexPageDTO> page = null;

        RespDTO<Page<StaticKnowledgeIndexPageDTO>> respDTO = cdssCoreClient.staticIndexPage(staticKnowledgeIndexPageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return page;
    }

    /**
     * 保存记录（新增or修改）
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean saveOrUpdateRecord(KlConceptStaticVO klConceptStaticVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        klConceptStaticVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.saveOrUpdateRecord(klConceptStaticVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return success;
    }

    /**
     * 保存病例（新增or修改）
     *
     * @param klClassicCaseVO
     * @return
     */
    public Boolean saveOrUpdateCase(KlClassicCaseVO klClassicCaseVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        klClassicCaseVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.saveOrUpdateCase(klClassicCaseVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return success;
    }

    /**
     * 启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeStatus(ChangeStatusVO changeStatusVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        changeStatusVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.changeStatus(changeStatusVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return success;
    }

    /**
     * 经典病例启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeCaseStatus(ChangeStatusVO changeStatusVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        changeStatusVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.changeCaseStatus(changeStatusVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return success;
    }

    /**
     * 根据术语id获取静态信息
     *
     * @param idVO
     * @return
     */
    public KlConceptStaticDTO getRecordById(IdVO idVO) {
        KlConceptStaticDTO klConceptStaticDTO = new KlConceptStaticDTO();
        RespDTO<KlConceptStaticDTO> respDTO = cdssCoreClient.getRecordById(idVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            klConceptStaticDTO = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return klConceptStaticDTO;
    }

    /**
     * 根据经典病例id获取病例信息
     *
     * @param idVO
     * @return
     */
    public KlClassicCaseDTO getCaseById(IdVO idVO) {
        KlClassicCaseDTO klClassicCaseDTO = new KlClassicCaseDTO();
        RespDTO<KlClassicCaseDTO> respDTO = cdssCoreClient.getCaseById(idVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            klClassicCaseDTO = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return klClassicCaseDTO;
    }

    /**
     * 是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean isExist(KlConceptStaticVO klConceptStaticVO) {
        Boolean exist = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.isExist(klConceptStaticVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            exist = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return exist;
    }

    /**
     * 收藏医学知识（静态知识）/经典病例
     * @param staticKnowledgeAndClassicCaseVO
     * @return
     */
    public Boolean collectionStaticKnowledgeAndClassicCase(StaticKnowledgeAndClassicCaseVO staticKnowledgeAndClassicCaseVO) {

        Boolean success = cdssCoreClient.collectionStaticKnowledgeAndClassicCase(staticKnowledgeAndClassicCaseVO);
        return success;
    }
}